/*!
    \file TMessageUnion_generated
    \author Kurt Stege
    Generated by the tool \c hpp_generator at Wed Sep 07 16:45:15 CEST 2005
*/

#ifndef NSigslot_TMessageUnion_generated_hpp_INCLUDEGUARD
#define NSigslot_TMessageUnion_generated_hpp_INCLUDEGUARD

/*
History:

(ks) = Kurt Stege, kurt-stege@online.de

2005-09-02 (ks):    File created.
2005-09-06 (ks):    A class tMessage cannot be stored in a union, when it
                    has non-trivial constructors, destructors or assignment
                    operator. Changed to emulate the union using placement new.
2005-09-06 (ks):    TEnpatcher and TDispatcher added.
2005-09-06 (ks):    current version, file still in development.
*/


#include <new>
#include <assert.h>

#include "Sigslot/TSignature.hpp"


namespace NSigslot {


// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
    TMessageUnion<n> is a template class to store one Sigslot message
    out of a set of signatures.

    The template \c TMessageUnion<n> gets n parameters,
    \c tMessage1 to \c tMessage0.
    These classes may be of any class of value kind, but are
    supposed to be of type NSigslot::TSignature<...>::tMessage
    for a different kind of signatures.

    The TMessageUnion is simply a union for all these given classes
    and has an addidional enum to store, which type actually is stored.
    \note This case TMessageUnion0<> is special. The signature
    gets zero arguments, alas, in C++ a template with zero
    arguments like TMessageUnion0<> is not valid. Instead, we
    have to remove the empty template argument list, and
    get an ordinary class TMessageUnion0.

    \note Even when TMessageUnion0 is quite useless, for it
    does not store any informations, it is implemented for the
    sake of generic programming. There might be code somewhere
    (e.g. C++ template classes) that might work for n=0 because
    of this otherwise useless class.

*/
class TMessageUnion0
{
    public:

    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    enum {numberOfParameters = 0};    //!< number of the parameters of the signal

    typedef TMessageUnion0 tMessageUnion;

    /*!
        This enum is used to identify one of the n types.
    */

    enum tMessageType
    {
        MT_none = 0 //!< no message is stored

    };

    enum tMessageType getMessageType() const
    {
        return m_messageType;
    }


    private:

    enum tMessageType m_messageType; //!< stores, of which type the currently stored messag is.

    /*!
        The member variable \c m_message stores the selected message.
        See \m_messageType to detect, which member of the union is valid.
    */

    union
    {
        // This union is empty on purpose.
    } m_message;


    public:

    TMessageUnion0()
    :
        m_messageType(MT_none)
    {
    }

    TMessageUnion0(const TMessageUnion0 &source)
    :
        m_messageType(MT_none)
    {
        *this = source;
    }
        
    TMessageUnion0 &operator= (const TMessageUnion0 &rhs)
    {
        if (this != &rhs)
        {
            setValue0();
            m_messageType = rhs.m_messageType;

        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose

        }

        }
        return *this;
    }

    ~TMessageUnion0()
    {
        setValue0();
    }


#if 0 // These constructors don't work, when the same type is used at several positions.

#endif


    /*!
        \c setValue0() is used to delete any stored element.
    */
    void setValue0()
    {
        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose

        }
        m_messageType = MT_none;
    }






}; // template class TMessageUnion0


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TEnpatcher0
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;
    typedef NSigslot::TSignature1<tMessageUnion> tOutputSignature;

    tOutputSignature::CSignal SIGNAL;

}; // template class TEnpatcher0


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TDispatcher0
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;

    void operator() (const tMessageUnion &mu)
    {
        switch (mu.getMessageType())
        {
            case tMessageUnion::MT_none:   break;  // empty on purpose

        }
    }


}; // template class TDispatcher0



// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
    TMessageUnion<n> is a template class to store one Sigslot message
    out of a set of signatures.

    The template \c TMessageUnion<n> gets n parameters,
    \c tMessage1 to \c tMessage1.
    These classes may be of any class of value kind, but are
    supposed to be of type NSigslot::TSignature<...>::tMessage
    for a different kind of signatures.

    The TMessageUnion is simply a union for all these given classes
    and has an addidional enum to store, which type actually is stored.

*/
template <typename TemplateParameter1>
class TMessageUnion1
{
    public:

    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tMessage1;    //!< type of the 1st/nd/rd/th signal parameter

    enum {numberOfParameters = 1};    //!< number of the parameters of the signal

    typedef TMessageUnion1<tMessage1> tMessageUnion;

    /*!
        This enum is used to identify one of the n types.
    */

    enum tMessageType
    {
        MT_none = 0 //!< no message is stored

      , MT_1 = 1    //!< type number 1 (tMessage1) is stored

    };

    enum tMessageType getMessageType() const
    {
        return m_messageType;
    }


    private:

    enum tMessageType m_messageType; //!< stores, of which type the currently stored messag is.

    /*!
        The member variable \c m_message stores the selected message.
        See \m_messageType to detect, which member of the union is valid.
    */

    union
    {
        int m_dummy0;         //!< to get a proper alignment...
        long m_dummy1;        //!< to get a proper alignment...
        double m_dummy2;      //!< to get a proper alignment...
        void *m_dummy3;       //!< to get a proper alignment...
        unsigned char m_par1[sizeof(tMessage1)];   //!< the 1st/nd/rd/th possible type to store
    } m_message;


    public:

    TMessageUnion1()
    :
        m_messageType(MT_none)
    {
    }

    TMessageUnion1(const TMessageUnion1 &source)
    :
        m_messageType(MT_none)
    {
        *this = source;
    }
        
    TMessageUnion1 &operator= (const TMessageUnion1 &rhs)
    {
        if (this != &rhs)
        {
            setValue0();
            m_messageType = rhs.m_messageType;

        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: new (m_message.m_par1)tMessage1(*(static_cast<const tMessage1 *>(static_cast<const void *>(rhs.m_message.m_par1)))); break;  // placement new
        }

        }
        return *this;
    }

    ~TMessageUnion1()
    {
        setValue0();
    }


#if 0 // These constructors don't work, when the same type is used at several positions.
    explicit TMessageUnion1 (tMessageType type, const tMessage1 &message)
    :
        m_messageType(type),
    {
        m_message.m_par1 = message;
    }

#endif


    /*!
        \c setValue0() is used to delete any stored element.
    */
    void setValue0()
    {
        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: (static_cast<tMessage1 *>(static_cast<void *>(m_message.m_par1)))->~TemplateParameter1(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
        }
        m_messageType = MT_none;
    }


    void setValue1(const tMessage1 &message)
    {
        setValue0();
        m_messageType = MT_1;
        new (m_message.m_par1)tMessage1(message); // placement new
    }


    const tMessage1 &getValue1() const
    {
        assert(m_messageType == MT_1);
        return *(static_cast<const tMessage1 *>(static_cast<const void *>(m_message.m_par1)));
    }


}; // template class TMessageUnion1


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TEnpatcher1
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;
    typedef NSigslot::TSignature1<tMessageUnion> tOutputSignature;

    tOutputSignature::CSignal SIGNAL;

    void SLOT_nr1 (const tMessageUnion::tMessage1 &msg)
    {
        tMessageUnion mu;
        mu.setValue1 (msg);
        SIGNAL (mu);
    }

}; // template class TEnpatcher1


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TDispatcher1
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;

    void operator() (const tMessageUnion &mu)
    {
        switch (mu.getMessageType())
        {
            case tMessageUnion::MT_none:   break;  // empty on purpose
            case tMessageUnion::MT_1:  SIGNAL_nr1(mu.getValue1()); break;
        }
    }

    NSigslot::TSignature1<tMessageUnion::tMessage1>::CSignal SIGNAL_nr1;
}; // template class TDispatcher1



// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
    TMessageUnion<n> is a template class to store one Sigslot message
    out of a set of signatures.

    The template \c TMessageUnion<n> gets n parameters,
    \c tMessage1 to \c tMessage2.
    These classes may be of any class of value kind, but are
    supposed to be of type NSigslot::TSignature<...>::tMessage
    for a different kind of signatures.

    The TMessageUnion is simply a union for all these given classes
    and has an addidional enum to store, which type actually is stored.

*/
template <typename TemplateParameter1, typename TemplateParameter2>
class TMessageUnion2
{
    public:

    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tMessage1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tMessage2;    //!< type of the 2st/nd/rd/th signal parameter

    enum {numberOfParameters = 2};    //!< number of the parameters of the signal

    typedef TMessageUnion2<tMessage1, tMessage2> tMessageUnion;

    /*!
        This enum is used to identify one of the n types.
    */

    enum tMessageType
    {
        MT_none = 0 //!< no message is stored

      , MT_1 = 1    //!< type number 1 (tMessage1) is stored
      , MT_2 = 2    //!< type number 2 (tMessage2) is stored

    };

    enum tMessageType getMessageType() const
    {
        return m_messageType;
    }


    private:

    enum tMessageType m_messageType; //!< stores, of which type the currently stored messag is.

    /*!
        The member variable \c m_message stores the selected message.
        See \m_messageType to detect, which member of the union is valid.
    */

    union
    {
        int m_dummy0;         //!< to get a proper alignment...
        long m_dummy1;        //!< to get a proper alignment...
        double m_dummy2;      //!< to get a proper alignment...
        void *m_dummy3;       //!< to get a proper alignment...
        unsigned char m_par1[sizeof(tMessage1)];   //!< the 1st/nd/rd/th possible type to store
        unsigned char m_par2[sizeof(tMessage2)];   //!< the 2st/nd/rd/th possible type to store
    } m_message;


    public:

    TMessageUnion2()
    :
        m_messageType(MT_none)
    {
    }

    TMessageUnion2(const TMessageUnion2 &source)
    :
        m_messageType(MT_none)
    {
        *this = source;
    }
        
    TMessageUnion2 &operator= (const TMessageUnion2 &rhs)
    {
        if (this != &rhs)
        {
            setValue0();
            m_messageType = rhs.m_messageType;

        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: new (m_message.m_par1)tMessage1(*(static_cast<const tMessage1 *>(static_cast<const void *>(rhs.m_message.m_par1)))); break;  // placement new
            case MT_2: new (m_message.m_par2)tMessage2(*(static_cast<const tMessage2 *>(static_cast<const void *>(rhs.m_message.m_par2)))); break;  // placement new
        }

        }
        return *this;
    }

    ~TMessageUnion2()
    {
        setValue0();
    }


#if 0 // These constructors don't work, when the same type is used at several positions.
    explicit TMessageUnion2 (tMessageType type, const tMessage1 &message)
    :
        m_messageType(type),
    {
        m_message.m_par1 = message;
    }

    explicit TMessageUnion2 (tMessageType type, const tMessage2 &message)
    :
        m_messageType(type),
    {
        m_message.m_par2 = message;
    }

#endif


    /*!
        \c setValue0() is used to delete any stored element.
    */
    void setValue0()
    {
        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: (static_cast<tMessage1 *>(static_cast<void *>(m_message.m_par1)))->~TemplateParameter1(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_2: (static_cast<tMessage2 *>(static_cast<void *>(m_message.m_par2)))->~TemplateParameter2(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
        }
        m_messageType = MT_none;
    }


    void setValue1(const tMessage1 &message)
    {
        setValue0();
        m_messageType = MT_1;
        new (m_message.m_par1)tMessage1(message); // placement new
    }

    void setValue2(const tMessage2 &message)
    {
        setValue0();
        m_messageType = MT_2;
        new (m_message.m_par2)tMessage2(message); // placement new
    }


    const tMessage1 &getValue1() const
    {
        assert(m_messageType == MT_1);
        return *(static_cast<const tMessage1 *>(static_cast<const void *>(m_message.m_par1)));
    }

    const tMessage2 &getValue2() const
    {
        assert(m_messageType == MT_2);
        return *(static_cast<const tMessage2 *>(static_cast<const void *>(m_message.m_par2)));
    }


}; // template class TMessageUnion2


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TEnpatcher2
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;
    typedef NSigslot::TSignature1<tMessageUnion> tOutputSignature;

    tOutputSignature::CSignal SIGNAL;

    void SLOT_nr1 (const tMessageUnion::tMessage1 &msg)
    {
        tMessageUnion mu;
        mu.setValue1 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr2 (const tMessageUnion::tMessage2 &msg)
    {
        tMessageUnion mu;
        mu.setValue2 (msg);
        SIGNAL (mu);
    }

}; // template class TEnpatcher2


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TDispatcher2
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;

    void operator() (const tMessageUnion &mu)
    {
        switch (mu.getMessageType())
        {
            case tMessageUnion::MT_none:   break;  // empty on purpose
            case tMessageUnion::MT_1:  SIGNAL_nr1(mu.getValue1()); break;
            case tMessageUnion::MT_2:  SIGNAL_nr2(mu.getValue2()); break;
        }
    }

    NSigslot::TSignature1<tMessageUnion::tMessage1>::CSignal SIGNAL_nr1;
    NSigslot::TSignature1<tMessageUnion::tMessage2>::CSignal SIGNAL_nr2;
}; // template class TDispatcher2



// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
    TMessageUnion<n> is a template class to store one Sigslot message
    out of a set of signatures.

    The template \c TMessageUnion<n> gets n parameters,
    \c tMessage1 to \c tMessage3.
    These classes may be of any class of value kind, but are
    supposed to be of type NSigslot::TSignature<...>::tMessage
    for a different kind of signatures.

    The TMessageUnion is simply a union for all these given classes
    and has an addidional enum to store, which type actually is stored.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3>
class TMessageUnion3
{
    public:

    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tMessage1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tMessage2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tMessage3;    //!< type of the 3st/nd/rd/th signal parameter

    enum {numberOfParameters = 3};    //!< number of the parameters of the signal

    typedef TMessageUnion3<tMessage1, tMessage2, tMessage3> tMessageUnion;

    /*!
        This enum is used to identify one of the n types.
    */

    enum tMessageType
    {
        MT_none = 0 //!< no message is stored

      , MT_1 = 1    //!< type number 1 (tMessage1) is stored
      , MT_2 = 2    //!< type number 2 (tMessage2) is stored
      , MT_3 = 3    //!< type number 3 (tMessage3) is stored

    };

    enum tMessageType getMessageType() const
    {
        return m_messageType;
    }


    private:

    enum tMessageType m_messageType; //!< stores, of which type the currently stored messag is.

    /*!
        The member variable \c m_message stores the selected message.
        See \m_messageType to detect, which member of the union is valid.
    */

    union
    {
        int m_dummy0;         //!< to get a proper alignment...
        long m_dummy1;        //!< to get a proper alignment...
        double m_dummy2;      //!< to get a proper alignment...
        void *m_dummy3;       //!< to get a proper alignment...
        unsigned char m_par1[sizeof(tMessage1)];   //!< the 1st/nd/rd/th possible type to store
        unsigned char m_par2[sizeof(tMessage2)];   //!< the 2st/nd/rd/th possible type to store
        unsigned char m_par3[sizeof(tMessage3)];   //!< the 3st/nd/rd/th possible type to store
    } m_message;


    public:

    TMessageUnion3()
    :
        m_messageType(MT_none)
    {
    }

    TMessageUnion3(const TMessageUnion3 &source)
    :
        m_messageType(MT_none)
    {
        *this = source;
    }
        
    TMessageUnion3 &operator= (const TMessageUnion3 &rhs)
    {
        if (this != &rhs)
        {
            setValue0();
            m_messageType = rhs.m_messageType;

        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: new (m_message.m_par1)tMessage1(*(static_cast<const tMessage1 *>(static_cast<const void *>(rhs.m_message.m_par1)))); break;  // placement new
            case MT_2: new (m_message.m_par2)tMessage2(*(static_cast<const tMessage2 *>(static_cast<const void *>(rhs.m_message.m_par2)))); break;  // placement new
            case MT_3: new (m_message.m_par3)tMessage3(*(static_cast<const tMessage3 *>(static_cast<const void *>(rhs.m_message.m_par3)))); break;  // placement new
        }

        }
        return *this;
    }

    ~TMessageUnion3()
    {
        setValue0();
    }


#if 0 // These constructors don't work, when the same type is used at several positions.
    explicit TMessageUnion3 (tMessageType type, const tMessage1 &message)
    :
        m_messageType(type),
    {
        m_message.m_par1 = message;
    }

    explicit TMessageUnion3 (tMessageType type, const tMessage2 &message)
    :
        m_messageType(type),
    {
        m_message.m_par2 = message;
    }

    explicit TMessageUnion3 (tMessageType type, const tMessage3 &message)
    :
        m_messageType(type),
    {
        m_message.m_par3 = message;
    }

#endif


    /*!
        \c setValue0() is used to delete any stored element.
    */
    void setValue0()
    {
        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: (static_cast<tMessage1 *>(static_cast<void *>(m_message.m_par1)))->~TemplateParameter1(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_2: (static_cast<tMessage2 *>(static_cast<void *>(m_message.m_par2)))->~TemplateParameter2(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_3: (static_cast<tMessage3 *>(static_cast<void *>(m_message.m_par3)))->~TemplateParameter3(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
        }
        m_messageType = MT_none;
    }


    void setValue1(const tMessage1 &message)
    {
        setValue0();
        m_messageType = MT_1;
        new (m_message.m_par1)tMessage1(message); // placement new
    }

    void setValue2(const tMessage2 &message)
    {
        setValue0();
        m_messageType = MT_2;
        new (m_message.m_par2)tMessage2(message); // placement new
    }

    void setValue3(const tMessage3 &message)
    {
        setValue0();
        m_messageType = MT_3;
        new (m_message.m_par3)tMessage3(message); // placement new
    }


    const tMessage1 &getValue1() const
    {
        assert(m_messageType == MT_1);
        return *(static_cast<const tMessage1 *>(static_cast<const void *>(m_message.m_par1)));
    }

    const tMessage2 &getValue2() const
    {
        assert(m_messageType == MT_2);
        return *(static_cast<const tMessage2 *>(static_cast<const void *>(m_message.m_par2)));
    }

    const tMessage3 &getValue3() const
    {
        assert(m_messageType == MT_3);
        return *(static_cast<const tMessage3 *>(static_cast<const void *>(m_message.m_par3)));
    }


}; // template class TMessageUnion3


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TEnpatcher3
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;
    typedef NSigslot::TSignature1<tMessageUnion> tOutputSignature;

    tOutputSignature::CSignal SIGNAL;

    void SLOT_nr1 (const tMessageUnion::tMessage1 &msg)
    {
        tMessageUnion mu;
        mu.setValue1 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr2 (const tMessageUnion::tMessage2 &msg)
    {
        tMessageUnion mu;
        mu.setValue2 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr3 (const tMessageUnion::tMessage3 &msg)
    {
        tMessageUnion mu;
        mu.setValue3 (msg);
        SIGNAL (mu);
    }

}; // template class TEnpatcher3


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TDispatcher3
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;

    void operator() (const tMessageUnion &mu)
    {
        switch (mu.getMessageType())
        {
            case tMessageUnion::MT_none:   break;  // empty on purpose
            case tMessageUnion::MT_1:  SIGNAL_nr1(mu.getValue1()); break;
            case tMessageUnion::MT_2:  SIGNAL_nr2(mu.getValue2()); break;
            case tMessageUnion::MT_3:  SIGNAL_nr3(mu.getValue3()); break;
        }
    }

    NSigslot::TSignature1<tMessageUnion::tMessage1>::CSignal SIGNAL_nr1;
    NSigslot::TSignature1<tMessageUnion::tMessage2>::CSignal SIGNAL_nr2;
    NSigslot::TSignature1<tMessageUnion::tMessage3>::CSignal SIGNAL_nr3;
}; // template class TDispatcher3



// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
    TMessageUnion<n> is a template class to store one Sigslot message
    out of a set of signatures.

    The template \c TMessageUnion<n> gets n parameters,
    \c tMessage1 to \c tMessage4.
    These classes may be of any class of value kind, but are
    supposed to be of type NSigslot::TSignature<...>::tMessage
    for a different kind of signatures.

    The TMessageUnion is simply a union for all these given classes
    and has an addidional enum to store, which type actually is stored.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3, typename TemplateParameter4>
class TMessageUnion4
{
    public:

    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tMessage1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tMessage2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tMessage3;    //!< type of the 3st/nd/rd/th signal parameter
    typedef TemplateParameter4 tMessage4;    //!< type of the 4st/nd/rd/th signal parameter

    enum {numberOfParameters = 4};    //!< number of the parameters of the signal

    typedef TMessageUnion4<tMessage1, tMessage2, tMessage3, tMessage4> tMessageUnion;

    /*!
        This enum is used to identify one of the n types.
    */

    enum tMessageType
    {
        MT_none = 0 //!< no message is stored

      , MT_1 = 1    //!< type number 1 (tMessage1) is stored
      , MT_2 = 2    //!< type number 2 (tMessage2) is stored
      , MT_3 = 3    //!< type number 3 (tMessage3) is stored
      , MT_4 = 4    //!< type number 4 (tMessage4) is stored

    };

    enum tMessageType getMessageType() const
    {
        return m_messageType;
    }


    private:

    enum tMessageType m_messageType; //!< stores, of which type the currently stored messag is.

    /*!
        The member variable \c m_message stores the selected message.
        See \m_messageType to detect, which member of the union is valid.
    */

    union
    {
        int m_dummy0;         //!< to get a proper alignment...
        long m_dummy1;        //!< to get a proper alignment...
        double m_dummy2;      //!< to get a proper alignment...
        void *m_dummy3;       //!< to get a proper alignment...
        unsigned char m_par1[sizeof(tMessage1)];   //!< the 1st/nd/rd/th possible type to store
        unsigned char m_par2[sizeof(tMessage2)];   //!< the 2st/nd/rd/th possible type to store
        unsigned char m_par3[sizeof(tMessage3)];   //!< the 3st/nd/rd/th possible type to store
        unsigned char m_par4[sizeof(tMessage4)];   //!< the 4st/nd/rd/th possible type to store
    } m_message;


    public:

    TMessageUnion4()
    :
        m_messageType(MT_none)
    {
    }

    TMessageUnion4(const TMessageUnion4 &source)
    :
        m_messageType(MT_none)
    {
        *this = source;
    }
        
    TMessageUnion4 &operator= (const TMessageUnion4 &rhs)
    {
        if (this != &rhs)
        {
            setValue0();
            m_messageType = rhs.m_messageType;

        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: new (m_message.m_par1)tMessage1(*(static_cast<const tMessage1 *>(static_cast<const void *>(rhs.m_message.m_par1)))); break;  // placement new
            case MT_2: new (m_message.m_par2)tMessage2(*(static_cast<const tMessage2 *>(static_cast<const void *>(rhs.m_message.m_par2)))); break;  // placement new
            case MT_3: new (m_message.m_par3)tMessage3(*(static_cast<const tMessage3 *>(static_cast<const void *>(rhs.m_message.m_par3)))); break;  // placement new
            case MT_4: new (m_message.m_par4)tMessage4(*(static_cast<const tMessage4 *>(static_cast<const void *>(rhs.m_message.m_par4)))); break;  // placement new
        }

        }
        return *this;
    }

    ~TMessageUnion4()
    {
        setValue0();
    }


#if 0 // These constructors don't work, when the same type is used at several positions.
    explicit TMessageUnion4 (tMessageType type, const tMessage1 &message)
    :
        m_messageType(type),
    {
        m_message.m_par1 = message;
    }

    explicit TMessageUnion4 (tMessageType type, const tMessage2 &message)
    :
        m_messageType(type),
    {
        m_message.m_par2 = message;
    }

    explicit TMessageUnion4 (tMessageType type, const tMessage3 &message)
    :
        m_messageType(type),
    {
        m_message.m_par3 = message;
    }

    explicit TMessageUnion4 (tMessageType type, const tMessage4 &message)
    :
        m_messageType(type),
    {
        m_message.m_par4 = message;
    }

#endif


    /*!
        \c setValue0() is used to delete any stored element.
    */
    void setValue0()
    {
        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: (static_cast<tMessage1 *>(static_cast<void *>(m_message.m_par1)))->~TemplateParameter1(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_2: (static_cast<tMessage2 *>(static_cast<void *>(m_message.m_par2)))->~TemplateParameter2(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_3: (static_cast<tMessage3 *>(static_cast<void *>(m_message.m_par3)))->~TemplateParameter3(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_4: (static_cast<tMessage4 *>(static_cast<void *>(m_message.m_par4)))->~TemplateParameter4(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
        }
        m_messageType = MT_none;
    }


    void setValue1(const tMessage1 &message)
    {
        setValue0();
        m_messageType = MT_1;
        new (m_message.m_par1)tMessage1(message); // placement new
    }

    void setValue2(const tMessage2 &message)
    {
        setValue0();
        m_messageType = MT_2;
        new (m_message.m_par2)tMessage2(message); // placement new
    }

    void setValue3(const tMessage3 &message)
    {
        setValue0();
        m_messageType = MT_3;
        new (m_message.m_par3)tMessage3(message); // placement new
    }

    void setValue4(const tMessage4 &message)
    {
        setValue0();
        m_messageType = MT_4;
        new (m_message.m_par4)tMessage4(message); // placement new
    }


    const tMessage1 &getValue1() const
    {
        assert(m_messageType == MT_1);
        return *(static_cast<const tMessage1 *>(static_cast<const void *>(m_message.m_par1)));
    }

    const tMessage2 &getValue2() const
    {
        assert(m_messageType == MT_2);
        return *(static_cast<const tMessage2 *>(static_cast<const void *>(m_message.m_par2)));
    }

    const tMessage3 &getValue3() const
    {
        assert(m_messageType == MT_3);
        return *(static_cast<const tMessage3 *>(static_cast<const void *>(m_message.m_par3)));
    }

    const tMessage4 &getValue4() const
    {
        assert(m_messageType == MT_4);
        return *(static_cast<const tMessage4 *>(static_cast<const void *>(m_message.m_par4)));
    }


}; // template class TMessageUnion4


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TEnpatcher4
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;
    typedef NSigslot::TSignature1<tMessageUnion> tOutputSignature;

    tOutputSignature::CSignal SIGNAL;

    void SLOT_nr1 (const tMessageUnion::tMessage1 &msg)
    {
        tMessageUnion mu;
        mu.setValue1 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr2 (const tMessageUnion::tMessage2 &msg)
    {
        tMessageUnion mu;
        mu.setValue2 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr3 (const tMessageUnion::tMessage3 &msg)
    {
        tMessageUnion mu;
        mu.setValue3 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr4 (const tMessageUnion::tMessage4 &msg)
    {
        tMessageUnion mu;
        mu.setValue4 (msg);
        SIGNAL (mu);
    }

}; // template class TEnpatcher4


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TDispatcher4
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;

    void operator() (const tMessageUnion &mu)
    {
        switch (mu.getMessageType())
        {
            case tMessageUnion::MT_none:   break;  // empty on purpose
            case tMessageUnion::MT_1:  SIGNAL_nr1(mu.getValue1()); break;
            case tMessageUnion::MT_2:  SIGNAL_nr2(mu.getValue2()); break;
            case tMessageUnion::MT_3:  SIGNAL_nr3(mu.getValue3()); break;
            case tMessageUnion::MT_4:  SIGNAL_nr4(mu.getValue4()); break;
        }
    }

    NSigslot::TSignature1<tMessageUnion::tMessage1>::CSignal SIGNAL_nr1;
    NSigslot::TSignature1<tMessageUnion::tMessage2>::CSignal SIGNAL_nr2;
    NSigslot::TSignature1<tMessageUnion::tMessage3>::CSignal SIGNAL_nr3;
    NSigslot::TSignature1<tMessageUnion::tMessage4>::CSignal SIGNAL_nr4;
}; // template class TDispatcher4



// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
    TMessageUnion<n> is a template class to store one Sigslot message
    out of a set of signatures.

    The template \c TMessageUnion<n> gets n parameters,
    \c tMessage1 to \c tMessage5.
    These classes may be of any class of value kind, but are
    supposed to be of type NSigslot::TSignature<...>::tMessage
    for a different kind of signatures.

    The TMessageUnion is simply a union for all these given classes
    and has an addidional enum to store, which type actually is stored.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3, typename TemplateParameter4, typename TemplateParameter5>
class TMessageUnion5
{
    public:

    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tMessage1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tMessage2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tMessage3;    //!< type of the 3st/nd/rd/th signal parameter
    typedef TemplateParameter4 tMessage4;    //!< type of the 4st/nd/rd/th signal parameter
    typedef TemplateParameter5 tMessage5;    //!< type of the 5st/nd/rd/th signal parameter

    enum {numberOfParameters = 5};    //!< number of the parameters of the signal

    typedef TMessageUnion5<tMessage1, tMessage2, tMessage3, tMessage4, tMessage5> tMessageUnion;

    /*!
        This enum is used to identify one of the n types.
    */

    enum tMessageType
    {
        MT_none = 0 //!< no message is stored

      , MT_1 = 1    //!< type number 1 (tMessage1) is stored
      , MT_2 = 2    //!< type number 2 (tMessage2) is stored
      , MT_3 = 3    //!< type number 3 (tMessage3) is stored
      , MT_4 = 4    //!< type number 4 (tMessage4) is stored
      , MT_5 = 5    //!< type number 5 (tMessage5) is stored

    };

    enum tMessageType getMessageType() const
    {
        return m_messageType;
    }


    private:

    enum tMessageType m_messageType; //!< stores, of which type the currently stored messag is.

    /*!
        The member variable \c m_message stores the selected message.
        See \m_messageType to detect, which member of the union is valid.
    */

    union
    {
        int m_dummy0;         //!< to get a proper alignment...
        long m_dummy1;        //!< to get a proper alignment...
        double m_dummy2;      //!< to get a proper alignment...
        void *m_dummy3;       //!< to get a proper alignment...
        unsigned char m_par1[sizeof(tMessage1)];   //!< the 1st/nd/rd/th possible type to store
        unsigned char m_par2[sizeof(tMessage2)];   //!< the 2st/nd/rd/th possible type to store
        unsigned char m_par3[sizeof(tMessage3)];   //!< the 3st/nd/rd/th possible type to store
        unsigned char m_par4[sizeof(tMessage4)];   //!< the 4st/nd/rd/th possible type to store
        unsigned char m_par5[sizeof(tMessage5)];   //!< the 5st/nd/rd/th possible type to store
    } m_message;


    public:

    TMessageUnion5()
    :
        m_messageType(MT_none)
    {
    }

    TMessageUnion5(const TMessageUnion5 &source)
    :
        m_messageType(MT_none)
    {
        *this = source;
    }
        
    TMessageUnion5 &operator= (const TMessageUnion5 &rhs)
    {
        if (this != &rhs)
        {
            setValue0();
            m_messageType = rhs.m_messageType;

        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: new (m_message.m_par1)tMessage1(*(static_cast<const tMessage1 *>(static_cast<const void *>(rhs.m_message.m_par1)))); break;  // placement new
            case MT_2: new (m_message.m_par2)tMessage2(*(static_cast<const tMessage2 *>(static_cast<const void *>(rhs.m_message.m_par2)))); break;  // placement new
            case MT_3: new (m_message.m_par3)tMessage3(*(static_cast<const tMessage3 *>(static_cast<const void *>(rhs.m_message.m_par3)))); break;  // placement new
            case MT_4: new (m_message.m_par4)tMessage4(*(static_cast<const tMessage4 *>(static_cast<const void *>(rhs.m_message.m_par4)))); break;  // placement new
            case MT_5: new (m_message.m_par5)tMessage5(*(static_cast<const tMessage5 *>(static_cast<const void *>(rhs.m_message.m_par5)))); break;  // placement new
        }

        }
        return *this;
    }

    ~TMessageUnion5()
    {
        setValue0();
    }


#if 0 // These constructors don't work, when the same type is used at several positions.
    explicit TMessageUnion5 (tMessageType type, const tMessage1 &message)
    :
        m_messageType(type),
    {
        m_message.m_par1 = message;
    }

    explicit TMessageUnion5 (tMessageType type, const tMessage2 &message)
    :
        m_messageType(type),
    {
        m_message.m_par2 = message;
    }

    explicit TMessageUnion5 (tMessageType type, const tMessage3 &message)
    :
        m_messageType(type),
    {
        m_message.m_par3 = message;
    }

    explicit TMessageUnion5 (tMessageType type, const tMessage4 &message)
    :
        m_messageType(type),
    {
        m_message.m_par4 = message;
    }

    explicit TMessageUnion5 (tMessageType type, const tMessage5 &message)
    :
        m_messageType(type),
    {
        m_message.m_par5 = message;
    }

#endif


    /*!
        \c setValue0() is used to delete any stored element.
    */
    void setValue0()
    {
        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: (static_cast<tMessage1 *>(static_cast<void *>(m_message.m_par1)))->~TemplateParameter1(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_2: (static_cast<tMessage2 *>(static_cast<void *>(m_message.m_par2)))->~TemplateParameter2(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_3: (static_cast<tMessage3 *>(static_cast<void *>(m_message.m_par3)))->~TemplateParameter3(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_4: (static_cast<tMessage4 *>(static_cast<void *>(m_message.m_par4)))->~TemplateParameter4(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_5: (static_cast<tMessage5 *>(static_cast<void *>(m_message.m_par5)))->~TemplateParameter5(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
        }
        m_messageType = MT_none;
    }


    void setValue1(const tMessage1 &message)
    {
        setValue0();
        m_messageType = MT_1;
        new (m_message.m_par1)tMessage1(message); // placement new
    }

    void setValue2(const tMessage2 &message)
    {
        setValue0();
        m_messageType = MT_2;
        new (m_message.m_par2)tMessage2(message); // placement new
    }

    void setValue3(const tMessage3 &message)
    {
        setValue0();
        m_messageType = MT_3;
        new (m_message.m_par3)tMessage3(message); // placement new
    }

    void setValue4(const tMessage4 &message)
    {
        setValue0();
        m_messageType = MT_4;
        new (m_message.m_par4)tMessage4(message); // placement new
    }

    void setValue5(const tMessage5 &message)
    {
        setValue0();
        m_messageType = MT_5;
        new (m_message.m_par5)tMessage5(message); // placement new
    }


    const tMessage1 &getValue1() const
    {
        assert(m_messageType == MT_1);
        return *(static_cast<const tMessage1 *>(static_cast<const void *>(m_message.m_par1)));
    }

    const tMessage2 &getValue2() const
    {
        assert(m_messageType == MT_2);
        return *(static_cast<const tMessage2 *>(static_cast<const void *>(m_message.m_par2)));
    }

    const tMessage3 &getValue3() const
    {
        assert(m_messageType == MT_3);
        return *(static_cast<const tMessage3 *>(static_cast<const void *>(m_message.m_par3)));
    }

    const tMessage4 &getValue4() const
    {
        assert(m_messageType == MT_4);
        return *(static_cast<const tMessage4 *>(static_cast<const void *>(m_message.m_par4)));
    }

    const tMessage5 &getValue5() const
    {
        assert(m_messageType == MT_5);
        return *(static_cast<const tMessage5 *>(static_cast<const void *>(m_message.m_par5)));
    }


}; // template class TMessageUnion5


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TEnpatcher5
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;
    typedef NSigslot::TSignature1<tMessageUnion> tOutputSignature;

    tOutputSignature::CSignal SIGNAL;

    void SLOT_nr1 (const tMessageUnion::tMessage1 &msg)
    {
        tMessageUnion mu;
        mu.setValue1 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr2 (const tMessageUnion::tMessage2 &msg)
    {
        tMessageUnion mu;
        mu.setValue2 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr3 (const tMessageUnion::tMessage3 &msg)
    {
        tMessageUnion mu;
        mu.setValue3 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr4 (const tMessageUnion::tMessage4 &msg)
    {
        tMessageUnion mu;
        mu.setValue4 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr5 (const tMessageUnion::tMessage5 &msg)
    {
        tMessageUnion mu;
        mu.setValue5 (msg);
        SIGNAL (mu);
    }

}; // template class TEnpatcher5


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TDispatcher5
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;

    void operator() (const tMessageUnion &mu)
    {
        switch (mu.getMessageType())
        {
            case tMessageUnion::MT_none:   break;  // empty on purpose
            case tMessageUnion::MT_1:  SIGNAL_nr1(mu.getValue1()); break;
            case tMessageUnion::MT_2:  SIGNAL_nr2(mu.getValue2()); break;
            case tMessageUnion::MT_3:  SIGNAL_nr3(mu.getValue3()); break;
            case tMessageUnion::MT_4:  SIGNAL_nr4(mu.getValue4()); break;
            case tMessageUnion::MT_5:  SIGNAL_nr5(mu.getValue5()); break;
        }
    }

    NSigslot::TSignature1<tMessageUnion::tMessage1>::CSignal SIGNAL_nr1;
    NSigslot::TSignature1<tMessageUnion::tMessage2>::CSignal SIGNAL_nr2;
    NSigslot::TSignature1<tMessageUnion::tMessage3>::CSignal SIGNAL_nr3;
    NSigslot::TSignature1<tMessageUnion::tMessage4>::CSignal SIGNAL_nr4;
    NSigslot::TSignature1<tMessageUnion::tMessage5>::CSignal SIGNAL_nr5;
}; // template class TDispatcher5



// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
    TMessageUnion<n> is a template class to store one Sigslot message
    out of a set of signatures.

    The template \c TMessageUnion<n> gets n parameters,
    \c tMessage1 to \c tMessage6.
    These classes may be of any class of value kind, but are
    supposed to be of type NSigslot::TSignature<...>::tMessage
    for a different kind of signatures.

    The TMessageUnion is simply a union for all these given classes
    and has an addidional enum to store, which type actually is stored.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3, typename TemplateParameter4, typename TemplateParameter5, typename TemplateParameter6>
class TMessageUnion6
{
    public:

    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tMessage1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tMessage2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tMessage3;    //!< type of the 3st/nd/rd/th signal parameter
    typedef TemplateParameter4 tMessage4;    //!< type of the 4st/nd/rd/th signal parameter
    typedef TemplateParameter5 tMessage5;    //!< type of the 5st/nd/rd/th signal parameter
    typedef TemplateParameter6 tMessage6;    //!< type of the 6st/nd/rd/th signal parameter

    enum {numberOfParameters = 6};    //!< number of the parameters of the signal

    typedef TMessageUnion6<tMessage1, tMessage2, tMessage3, tMessage4, tMessage5, tMessage6> tMessageUnion;

    /*!
        This enum is used to identify one of the n types.
    */

    enum tMessageType
    {
        MT_none = 0 //!< no message is stored

      , MT_1 = 1    //!< type number 1 (tMessage1) is stored
      , MT_2 = 2    //!< type number 2 (tMessage2) is stored
      , MT_3 = 3    //!< type number 3 (tMessage3) is stored
      , MT_4 = 4    //!< type number 4 (tMessage4) is stored
      , MT_5 = 5    //!< type number 5 (tMessage5) is stored
      , MT_6 = 6    //!< type number 6 (tMessage6) is stored

    };

    enum tMessageType getMessageType() const
    {
        return m_messageType;
    }


    private:

    enum tMessageType m_messageType; //!< stores, of which type the currently stored messag is.

    /*!
        The member variable \c m_message stores the selected message.
        See \m_messageType to detect, which member of the union is valid.
    */

    union
    {
        int m_dummy0;         //!< to get a proper alignment...
        long m_dummy1;        //!< to get a proper alignment...
        double m_dummy2;      //!< to get a proper alignment...
        void *m_dummy3;       //!< to get a proper alignment...
        unsigned char m_par1[sizeof(tMessage1)];   //!< the 1st/nd/rd/th possible type to store
        unsigned char m_par2[sizeof(tMessage2)];   //!< the 2st/nd/rd/th possible type to store
        unsigned char m_par3[sizeof(tMessage3)];   //!< the 3st/nd/rd/th possible type to store
        unsigned char m_par4[sizeof(tMessage4)];   //!< the 4st/nd/rd/th possible type to store
        unsigned char m_par5[sizeof(tMessage5)];   //!< the 5st/nd/rd/th possible type to store
        unsigned char m_par6[sizeof(tMessage6)];   //!< the 6st/nd/rd/th possible type to store
    } m_message;


    public:

    TMessageUnion6()
    :
        m_messageType(MT_none)
    {
    }

    TMessageUnion6(const TMessageUnion6 &source)
    :
        m_messageType(MT_none)
    {
        *this = source;
    }
        
    TMessageUnion6 &operator= (const TMessageUnion6 &rhs)
    {
        if (this != &rhs)
        {
            setValue0();
            m_messageType = rhs.m_messageType;

        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: new (m_message.m_par1)tMessage1(*(static_cast<const tMessage1 *>(static_cast<const void *>(rhs.m_message.m_par1)))); break;  // placement new
            case MT_2: new (m_message.m_par2)tMessage2(*(static_cast<const tMessage2 *>(static_cast<const void *>(rhs.m_message.m_par2)))); break;  // placement new
            case MT_3: new (m_message.m_par3)tMessage3(*(static_cast<const tMessage3 *>(static_cast<const void *>(rhs.m_message.m_par3)))); break;  // placement new
            case MT_4: new (m_message.m_par4)tMessage4(*(static_cast<const tMessage4 *>(static_cast<const void *>(rhs.m_message.m_par4)))); break;  // placement new
            case MT_5: new (m_message.m_par5)tMessage5(*(static_cast<const tMessage5 *>(static_cast<const void *>(rhs.m_message.m_par5)))); break;  // placement new
            case MT_6: new (m_message.m_par6)tMessage6(*(static_cast<const tMessage6 *>(static_cast<const void *>(rhs.m_message.m_par6)))); break;  // placement new
        }

        }
        return *this;
    }

    ~TMessageUnion6()
    {
        setValue0();
    }


#if 0 // These constructors don't work, when the same type is used at several positions.
    explicit TMessageUnion6 (tMessageType type, const tMessage1 &message)
    :
        m_messageType(type),
    {
        m_message.m_par1 = message;
    }

    explicit TMessageUnion6 (tMessageType type, const tMessage2 &message)
    :
        m_messageType(type),
    {
        m_message.m_par2 = message;
    }

    explicit TMessageUnion6 (tMessageType type, const tMessage3 &message)
    :
        m_messageType(type),
    {
        m_message.m_par3 = message;
    }

    explicit TMessageUnion6 (tMessageType type, const tMessage4 &message)
    :
        m_messageType(type),
    {
        m_message.m_par4 = message;
    }

    explicit TMessageUnion6 (tMessageType type, const tMessage5 &message)
    :
        m_messageType(type),
    {
        m_message.m_par5 = message;
    }

    explicit TMessageUnion6 (tMessageType type, const tMessage6 &message)
    :
        m_messageType(type),
    {
        m_message.m_par6 = message;
    }

#endif


    /*!
        \c setValue0() is used to delete any stored element.
    */
    void setValue0()
    {
        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: (static_cast<tMessage1 *>(static_cast<void *>(m_message.m_par1)))->~TemplateParameter1(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_2: (static_cast<tMessage2 *>(static_cast<void *>(m_message.m_par2)))->~TemplateParameter2(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_3: (static_cast<tMessage3 *>(static_cast<void *>(m_message.m_par3)))->~TemplateParameter3(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_4: (static_cast<tMessage4 *>(static_cast<void *>(m_message.m_par4)))->~TemplateParameter4(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_5: (static_cast<tMessage5 *>(static_cast<void *>(m_message.m_par5)))->~TemplateParameter5(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_6: (static_cast<tMessage6 *>(static_cast<void *>(m_message.m_par6)))->~TemplateParameter6(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
        }
        m_messageType = MT_none;
    }


    void setValue1(const tMessage1 &message)
    {
        setValue0();
        m_messageType = MT_1;
        new (m_message.m_par1)tMessage1(message); // placement new
    }

    void setValue2(const tMessage2 &message)
    {
        setValue0();
        m_messageType = MT_2;
        new (m_message.m_par2)tMessage2(message); // placement new
    }

    void setValue3(const tMessage3 &message)
    {
        setValue0();
        m_messageType = MT_3;
        new (m_message.m_par3)tMessage3(message); // placement new
    }

    void setValue4(const tMessage4 &message)
    {
        setValue0();
        m_messageType = MT_4;
        new (m_message.m_par4)tMessage4(message); // placement new
    }

    void setValue5(const tMessage5 &message)
    {
        setValue0();
        m_messageType = MT_5;
        new (m_message.m_par5)tMessage5(message); // placement new
    }

    void setValue6(const tMessage6 &message)
    {
        setValue0();
        m_messageType = MT_6;
        new (m_message.m_par6)tMessage6(message); // placement new
    }


    const tMessage1 &getValue1() const
    {
        assert(m_messageType == MT_1);
        return *(static_cast<const tMessage1 *>(static_cast<const void *>(m_message.m_par1)));
    }

    const tMessage2 &getValue2() const
    {
        assert(m_messageType == MT_2);
        return *(static_cast<const tMessage2 *>(static_cast<const void *>(m_message.m_par2)));
    }

    const tMessage3 &getValue3() const
    {
        assert(m_messageType == MT_3);
        return *(static_cast<const tMessage3 *>(static_cast<const void *>(m_message.m_par3)));
    }

    const tMessage4 &getValue4() const
    {
        assert(m_messageType == MT_4);
        return *(static_cast<const tMessage4 *>(static_cast<const void *>(m_message.m_par4)));
    }

    const tMessage5 &getValue5() const
    {
        assert(m_messageType == MT_5);
        return *(static_cast<const tMessage5 *>(static_cast<const void *>(m_message.m_par5)));
    }

    const tMessage6 &getValue6() const
    {
        assert(m_messageType == MT_6);
        return *(static_cast<const tMessage6 *>(static_cast<const void *>(m_message.m_par6)));
    }


}; // template class TMessageUnion6


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TEnpatcher6
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;
    typedef NSigslot::TSignature1<tMessageUnion> tOutputSignature;

    tOutputSignature::CSignal SIGNAL;

    void SLOT_nr1 (const tMessageUnion::tMessage1 &msg)
    {
        tMessageUnion mu;
        mu.setValue1 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr2 (const tMessageUnion::tMessage2 &msg)
    {
        tMessageUnion mu;
        mu.setValue2 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr3 (const tMessageUnion::tMessage3 &msg)
    {
        tMessageUnion mu;
        mu.setValue3 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr4 (const tMessageUnion::tMessage4 &msg)
    {
        tMessageUnion mu;
        mu.setValue4 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr5 (const tMessageUnion::tMessage5 &msg)
    {
        tMessageUnion mu;
        mu.setValue5 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr6 (const tMessageUnion::tMessage6 &msg)
    {
        tMessageUnion mu;
        mu.setValue6 (msg);
        SIGNAL (mu);
    }

}; // template class TEnpatcher6


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TDispatcher6
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;

    void operator() (const tMessageUnion &mu)
    {
        switch (mu.getMessageType())
        {
            case tMessageUnion::MT_none:   break;  // empty on purpose
            case tMessageUnion::MT_1:  SIGNAL_nr1(mu.getValue1()); break;
            case tMessageUnion::MT_2:  SIGNAL_nr2(mu.getValue2()); break;
            case tMessageUnion::MT_3:  SIGNAL_nr3(mu.getValue3()); break;
            case tMessageUnion::MT_4:  SIGNAL_nr4(mu.getValue4()); break;
            case tMessageUnion::MT_5:  SIGNAL_nr5(mu.getValue5()); break;
            case tMessageUnion::MT_6:  SIGNAL_nr6(mu.getValue6()); break;
        }
    }

    NSigslot::TSignature1<tMessageUnion::tMessage1>::CSignal SIGNAL_nr1;
    NSigslot::TSignature1<tMessageUnion::tMessage2>::CSignal SIGNAL_nr2;
    NSigslot::TSignature1<tMessageUnion::tMessage3>::CSignal SIGNAL_nr3;
    NSigslot::TSignature1<tMessageUnion::tMessage4>::CSignal SIGNAL_nr4;
    NSigslot::TSignature1<tMessageUnion::tMessage5>::CSignal SIGNAL_nr5;
    NSigslot::TSignature1<tMessageUnion::tMessage6>::CSignal SIGNAL_nr6;
}; // template class TDispatcher6



// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
    TMessageUnion<n> is a template class to store one Sigslot message
    out of a set of signatures.

    The template \c TMessageUnion<n> gets n parameters,
    \c tMessage1 to \c tMessage7.
    These classes may be of any class of value kind, but are
    supposed to be of type NSigslot::TSignature<...>::tMessage
    for a different kind of signatures.

    The TMessageUnion is simply a union for all these given classes
    and has an addidional enum to store, which type actually is stored.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3, typename TemplateParameter4, typename TemplateParameter5, typename TemplateParameter6, typename TemplateParameter7>
class TMessageUnion7
{
    public:

    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tMessage1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tMessage2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tMessage3;    //!< type of the 3st/nd/rd/th signal parameter
    typedef TemplateParameter4 tMessage4;    //!< type of the 4st/nd/rd/th signal parameter
    typedef TemplateParameter5 tMessage5;    //!< type of the 5st/nd/rd/th signal parameter
    typedef TemplateParameter6 tMessage6;    //!< type of the 6st/nd/rd/th signal parameter
    typedef TemplateParameter7 tMessage7;    //!< type of the 7st/nd/rd/th signal parameter

    enum {numberOfParameters = 7};    //!< number of the parameters of the signal

    typedef TMessageUnion7<tMessage1, tMessage2, tMessage3, tMessage4, tMessage5, tMessage6, tMessage7> tMessageUnion;

    /*!
        This enum is used to identify one of the n types.
    */

    enum tMessageType
    {
        MT_none = 0 //!< no message is stored

      , MT_1 = 1    //!< type number 1 (tMessage1) is stored
      , MT_2 = 2    //!< type number 2 (tMessage2) is stored
      , MT_3 = 3    //!< type number 3 (tMessage3) is stored
      , MT_4 = 4    //!< type number 4 (tMessage4) is stored
      , MT_5 = 5    //!< type number 5 (tMessage5) is stored
      , MT_6 = 6    //!< type number 6 (tMessage6) is stored
      , MT_7 = 7    //!< type number 7 (tMessage7) is stored

    };

    enum tMessageType getMessageType() const
    {
        return m_messageType;
    }


    private:

    enum tMessageType m_messageType; //!< stores, of which type the currently stored messag is.

    /*!
        The member variable \c m_message stores the selected message.
        See \m_messageType to detect, which member of the union is valid.
    */

    union
    {
        int m_dummy0;         //!< to get a proper alignment...
        long m_dummy1;        //!< to get a proper alignment...
        double m_dummy2;      //!< to get a proper alignment...
        void *m_dummy3;       //!< to get a proper alignment...
        unsigned char m_par1[sizeof(tMessage1)];   //!< the 1st/nd/rd/th possible type to store
        unsigned char m_par2[sizeof(tMessage2)];   //!< the 2st/nd/rd/th possible type to store
        unsigned char m_par3[sizeof(tMessage3)];   //!< the 3st/nd/rd/th possible type to store
        unsigned char m_par4[sizeof(tMessage4)];   //!< the 4st/nd/rd/th possible type to store
        unsigned char m_par5[sizeof(tMessage5)];   //!< the 5st/nd/rd/th possible type to store
        unsigned char m_par6[sizeof(tMessage6)];   //!< the 6st/nd/rd/th possible type to store
        unsigned char m_par7[sizeof(tMessage7)];   //!< the 7st/nd/rd/th possible type to store
    } m_message;


    public:

    TMessageUnion7()
    :
        m_messageType(MT_none)
    {
    }

    TMessageUnion7(const TMessageUnion7 &source)
    :
        m_messageType(MT_none)
    {
        *this = source;
    }
        
    TMessageUnion7 &operator= (const TMessageUnion7 &rhs)
    {
        if (this != &rhs)
        {
            setValue0();
            m_messageType = rhs.m_messageType;

        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: new (m_message.m_par1)tMessage1(*(static_cast<const tMessage1 *>(static_cast<const void *>(rhs.m_message.m_par1)))); break;  // placement new
            case MT_2: new (m_message.m_par2)tMessage2(*(static_cast<const tMessage2 *>(static_cast<const void *>(rhs.m_message.m_par2)))); break;  // placement new
            case MT_3: new (m_message.m_par3)tMessage3(*(static_cast<const tMessage3 *>(static_cast<const void *>(rhs.m_message.m_par3)))); break;  // placement new
            case MT_4: new (m_message.m_par4)tMessage4(*(static_cast<const tMessage4 *>(static_cast<const void *>(rhs.m_message.m_par4)))); break;  // placement new
            case MT_5: new (m_message.m_par5)tMessage5(*(static_cast<const tMessage5 *>(static_cast<const void *>(rhs.m_message.m_par5)))); break;  // placement new
            case MT_6: new (m_message.m_par6)tMessage6(*(static_cast<const tMessage6 *>(static_cast<const void *>(rhs.m_message.m_par6)))); break;  // placement new
            case MT_7: new (m_message.m_par7)tMessage7(*(static_cast<const tMessage7 *>(static_cast<const void *>(rhs.m_message.m_par7)))); break;  // placement new
        }

        }
        return *this;
    }

    ~TMessageUnion7()
    {
        setValue0();
    }


#if 0 // These constructors don't work, when the same type is used at several positions.
    explicit TMessageUnion7 (tMessageType type, const tMessage1 &message)
    :
        m_messageType(type),
    {
        m_message.m_par1 = message;
    }

    explicit TMessageUnion7 (tMessageType type, const tMessage2 &message)
    :
        m_messageType(type),
    {
        m_message.m_par2 = message;
    }

    explicit TMessageUnion7 (tMessageType type, const tMessage3 &message)
    :
        m_messageType(type),
    {
        m_message.m_par3 = message;
    }

    explicit TMessageUnion7 (tMessageType type, const tMessage4 &message)
    :
        m_messageType(type),
    {
        m_message.m_par4 = message;
    }

    explicit TMessageUnion7 (tMessageType type, const tMessage5 &message)
    :
        m_messageType(type),
    {
        m_message.m_par5 = message;
    }

    explicit TMessageUnion7 (tMessageType type, const tMessage6 &message)
    :
        m_messageType(type),
    {
        m_message.m_par6 = message;
    }

    explicit TMessageUnion7 (tMessageType type, const tMessage7 &message)
    :
        m_messageType(type),
    {
        m_message.m_par7 = message;
    }

#endif


    /*!
        \c setValue0() is used to delete any stored element.
    */
    void setValue0()
    {
        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: (static_cast<tMessage1 *>(static_cast<void *>(m_message.m_par1)))->~TemplateParameter1(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_2: (static_cast<tMessage2 *>(static_cast<void *>(m_message.m_par2)))->~TemplateParameter2(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_3: (static_cast<tMessage3 *>(static_cast<void *>(m_message.m_par3)))->~TemplateParameter3(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_4: (static_cast<tMessage4 *>(static_cast<void *>(m_message.m_par4)))->~TemplateParameter4(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_5: (static_cast<tMessage5 *>(static_cast<void *>(m_message.m_par5)))->~TemplateParameter5(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_6: (static_cast<tMessage6 *>(static_cast<void *>(m_message.m_par6)))->~TemplateParameter6(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_7: (static_cast<tMessage7 *>(static_cast<void *>(m_message.m_par7)))->~TemplateParameter7(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
        }
        m_messageType = MT_none;
    }


    void setValue1(const tMessage1 &message)
    {
        setValue0();
        m_messageType = MT_1;
        new (m_message.m_par1)tMessage1(message); // placement new
    }

    void setValue2(const tMessage2 &message)
    {
        setValue0();
        m_messageType = MT_2;
        new (m_message.m_par2)tMessage2(message); // placement new
    }

    void setValue3(const tMessage3 &message)
    {
        setValue0();
        m_messageType = MT_3;
        new (m_message.m_par3)tMessage3(message); // placement new
    }

    void setValue4(const tMessage4 &message)
    {
        setValue0();
        m_messageType = MT_4;
        new (m_message.m_par4)tMessage4(message); // placement new
    }

    void setValue5(const tMessage5 &message)
    {
        setValue0();
        m_messageType = MT_5;
        new (m_message.m_par5)tMessage5(message); // placement new
    }

    void setValue6(const tMessage6 &message)
    {
        setValue0();
        m_messageType = MT_6;
        new (m_message.m_par6)tMessage6(message); // placement new
    }

    void setValue7(const tMessage7 &message)
    {
        setValue0();
        m_messageType = MT_7;
        new (m_message.m_par7)tMessage7(message); // placement new
    }


    const tMessage1 &getValue1() const
    {
        assert(m_messageType == MT_1);
        return *(static_cast<const tMessage1 *>(static_cast<const void *>(m_message.m_par1)));
    }

    const tMessage2 &getValue2() const
    {
        assert(m_messageType == MT_2);
        return *(static_cast<const tMessage2 *>(static_cast<const void *>(m_message.m_par2)));
    }

    const tMessage3 &getValue3() const
    {
        assert(m_messageType == MT_3);
        return *(static_cast<const tMessage3 *>(static_cast<const void *>(m_message.m_par3)));
    }

    const tMessage4 &getValue4() const
    {
        assert(m_messageType == MT_4);
        return *(static_cast<const tMessage4 *>(static_cast<const void *>(m_message.m_par4)));
    }

    const tMessage5 &getValue5() const
    {
        assert(m_messageType == MT_5);
        return *(static_cast<const tMessage5 *>(static_cast<const void *>(m_message.m_par5)));
    }

    const tMessage6 &getValue6() const
    {
        assert(m_messageType == MT_6);
        return *(static_cast<const tMessage6 *>(static_cast<const void *>(m_message.m_par6)));
    }

    const tMessage7 &getValue7() const
    {
        assert(m_messageType == MT_7);
        return *(static_cast<const tMessage7 *>(static_cast<const void *>(m_message.m_par7)));
    }


}; // template class TMessageUnion7


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TEnpatcher7
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;
    typedef NSigslot::TSignature1<tMessageUnion> tOutputSignature;

    tOutputSignature::CSignal SIGNAL;

    void SLOT_nr1 (const tMessageUnion::tMessage1 &msg)
    {
        tMessageUnion mu;
        mu.setValue1 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr2 (const tMessageUnion::tMessage2 &msg)
    {
        tMessageUnion mu;
        mu.setValue2 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr3 (const tMessageUnion::tMessage3 &msg)
    {
        tMessageUnion mu;
        mu.setValue3 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr4 (const tMessageUnion::tMessage4 &msg)
    {
        tMessageUnion mu;
        mu.setValue4 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr5 (const tMessageUnion::tMessage5 &msg)
    {
        tMessageUnion mu;
        mu.setValue5 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr6 (const tMessageUnion::tMessage6 &msg)
    {
        tMessageUnion mu;
        mu.setValue6 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr7 (const tMessageUnion::tMessage7 &msg)
    {
        tMessageUnion mu;
        mu.setValue7 (msg);
        SIGNAL (mu);
    }

}; // template class TEnpatcher7


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TDispatcher7
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;

    void operator() (const tMessageUnion &mu)
    {
        switch (mu.getMessageType())
        {
            case tMessageUnion::MT_none:   break;  // empty on purpose
            case tMessageUnion::MT_1:  SIGNAL_nr1(mu.getValue1()); break;
            case tMessageUnion::MT_2:  SIGNAL_nr2(mu.getValue2()); break;
            case tMessageUnion::MT_3:  SIGNAL_nr3(mu.getValue3()); break;
            case tMessageUnion::MT_4:  SIGNAL_nr4(mu.getValue4()); break;
            case tMessageUnion::MT_5:  SIGNAL_nr5(mu.getValue5()); break;
            case tMessageUnion::MT_6:  SIGNAL_nr6(mu.getValue6()); break;
            case tMessageUnion::MT_7:  SIGNAL_nr7(mu.getValue7()); break;
        }
    }

    NSigslot::TSignature1<tMessageUnion::tMessage1>::CSignal SIGNAL_nr1;
    NSigslot::TSignature1<tMessageUnion::tMessage2>::CSignal SIGNAL_nr2;
    NSigslot::TSignature1<tMessageUnion::tMessage3>::CSignal SIGNAL_nr3;
    NSigslot::TSignature1<tMessageUnion::tMessage4>::CSignal SIGNAL_nr4;
    NSigslot::TSignature1<tMessageUnion::tMessage5>::CSignal SIGNAL_nr5;
    NSigslot::TSignature1<tMessageUnion::tMessage6>::CSignal SIGNAL_nr6;
    NSigslot::TSignature1<tMessageUnion::tMessage7>::CSignal SIGNAL_nr7;
}; // template class TDispatcher7



// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
    TMessageUnion<n> is a template class to store one Sigslot message
    out of a set of signatures.

    The template \c TMessageUnion<n> gets n parameters,
    \c tMessage1 to \c tMessage8.
    These classes may be of any class of value kind, but are
    supposed to be of type NSigslot::TSignature<...>::tMessage
    for a different kind of signatures.

    The TMessageUnion is simply a union for all these given classes
    and has an addidional enum to store, which type actually is stored.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3, typename TemplateParameter4, typename TemplateParameter5, typename TemplateParameter6, typename TemplateParameter7, typename TemplateParameter8>
class TMessageUnion8
{
    public:

    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tMessage1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tMessage2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tMessage3;    //!< type of the 3st/nd/rd/th signal parameter
    typedef TemplateParameter4 tMessage4;    //!< type of the 4st/nd/rd/th signal parameter
    typedef TemplateParameter5 tMessage5;    //!< type of the 5st/nd/rd/th signal parameter
    typedef TemplateParameter6 tMessage6;    //!< type of the 6st/nd/rd/th signal parameter
    typedef TemplateParameter7 tMessage7;    //!< type of the 7st/nd/rd/th signal parameter
    typedef TemplateParameter8 tMessage8;    //!< type of the 8st/nd/rd/th signal parameter

    enum {numberOfParameters = 8};    //!< number of the parameters of the signal

    typedef TMessageUnion8<tMessage1, tMessage2, tMessage3, tMessage4, tMessage5, tMessage6, tMessage7, tMessage8> tMessageUnion;

    /*!
        This enum is used to identify one of the n types.
    */

    enum tMessageType
    {
        MT_none = 0 //!< no message is stored

      , MT_1 = 1    //!< type number 1 (tMessage1) is stored
      , MT_2 = 2    //!< type number 2 (tMessage2) is stored
      , MT_3 = 3    //!< type number 3 (tMessage3) is stored
      , MT_4 = 4    //!< type number 4 (tMessage4) is stored
      , MT_5 = 5    //!< type number 5 (tMessage5) is stored
      , MT_6 = 6    //!< type number 6 (tMessage6) is stored
      , MT_7 = 7    //!< type number 7 (tMessage7) is stored
      , MT_8 = 8    //!< type number 8 (tMessage8) is stored

    };

    enum tMessageType getMessageType() const
    {
        return m_messageType;
    }


    private:

    enum tMessageType m_messageType; //!< stores, of which type the currently stored messag is.

    /*!
        The member variable \c m_message stores the selected message.
        See \m_messageType to detect, which member of the union is valid.
    */

    union
    {
        int m_dummy0;         //!< to get a proper alignment...
        long m_dummy1;        //!< to get a proper alignment...
        double m_dummy2;      //!< to get a proper alignment...
        void *m_dummy3;       //!< to get a proper alignment...
        unsigned char m_par1[sizeof(tMessage1)];   //!< the 1st/nd/rd/th possible type to store
        unsigned char m_par2[sizeof(tMessage2)];   //!< the 2st/nd/rd/th possible type to store
        unsigned char m_par3[sizeof(tMessage3)];   //!< the 3st/nd/rd/th possible type to store
        unsigned char m_par4[sizeof(tMessage4)];   //!< the 4st/nd/rd/th possible type to store
        unsigned char m_par5[sizeof(tMessage5)];   //!< the 5st/nd/rd/th possible type to store
        unsigned char m_par6[sizeof(tMessage6)];   //!< the 6st/nd/rd/th possible type to store
        unsigned char m_par7[sizeof(tMessage7)];   //!< the 7st/nd/rd/th possible type to store
        unsigned char m_par8[sizeof(tMessage8)];   //!< the 8st/nd/rd/th possible type to store
    } m_message;


    public:

    TMessageUnion8()
    :
        m_messageType(MT_none)
    {
    }

    TMessageUnion8(const TMessageUnion8 &source)
    :
        m_messageType(MT_none)
    {
        *this = source;
    }
        
    TMessageUnion8 &operator= (const TMessageUnion8 &rhs)
    {
        if (this != &rhs)
        {
            setValue0();
            m_messageType = rhs.m_messageType;

        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: new (m_message.m_par1)tMessage1(*(static_cast<const tMessage1 *>(static_cast<const void *>(rhs.m_message.m_par1)))); break;  // placement new
            case MT_2: new (m_message.m_par2)tMessage2(*(static_cast<const tMessage2 *>(static_cast<const void *>(rhs.m_message.m_par2)))); break;  // placement new
            case MT_3: new (m_message.m_par3)tMessage3(*(static_cast<const tMessage3 *>(static_cast<const void *>(rhs.m_message.m_par3)))); break;  // placement new
            case MT_4: new (m_message.m_par4)tMessage4(*(static_cast<const tMessage4 *>(static_cast<const void *>(rhs.m_message.m_par4)))); break;  // placement new
            case MT_5: new (m_message.m_par5)tMessage5(*(static_cast<const tMessage5 *>(static_cast<const void *>(rhs.m_message.m_par5)))); break;  // placement new
            case MT_6: new (m_message.m_par6)tMessage6(*(static_cast<const tMessage6 *>(static_cast<const void *>(rhs.m_message.m_par6)))); break;  // placement new
            case MT_7: new (m_message.m_par7)tMessage7(*(static_cast<const tMessage7 *>(static_cast<const void *>(rhs.m_message.m_par7)))); break;  // placement new
            case MT_8: new (m_message.m_par8)tMessage8(*(static_cast<const tMessage8 *>(static_cast<const void *>(rhs.m_message.m_par8)))); break;  // placement new
        }

        }
        return *this;
    }

    ~TMessageUnion8()
    {
        setValue0();
    }


#if 0 // These constructors don't work, when the same type is used at several positions.
    explicit TMessageUnion8 (tMessageType type, const tMessage1 &message)
    :
        m_messageType(type),
    {
        m_message.m_par1 = message;
    }

    explicit TMessageUnion8 (tMessageType type, const tMessage2 &message)
    :
        m_messageType(type),
    {
        m_message.m_par2 = message;
    }

    explicit TMessageUnion8 (tMessageType type, const tMessage3 &message)
    :
        m_messageType(type),
    {
        m_message.m_par3 = message;
    }

    explicit TMessageUnion8 (tMessageType type, const tMessage4 &message)
    :
        m_messageType(type),
    {
        m_message.m_par4 = message;
    }

    explicit TMessageUnion8 (tMessageType type, const tMessage5 &message)
    :
        m_messageType(type),
    {
        m_message.m_par5 = message;
    }

    explicit TMessageUnion8 (tMessageType type, const tMessage6 &message)
    :
        m_messageType(type),
    {
        m_message.m_par6 = message;
    }

    explicit TMessageUnion8 (tMessageType type, const tMessage7 &message)
    :
        m_messageType(type),
    {
        m_message.m_par7 = message;
    }

    explicit TMessageUnion8 (tMessageType type, const tMessage8 &message)
    :
        m_messageType(type),
    {
        m_message.m_par8 = message;
    }

#endif


    /*!
        \c setValue0() is used to delete any stored element.
    */
    void setValue0()
    {
        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: (static_cast<tMessage1 *>(static_cast<void *>(m_message.m_par1)))->~TemplateParameter1(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_2: (static_cast<tMessage2 *>(static_cast<void *>(m_message.m_par2)))->~TemplateParameter2(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_3: (static_cast<tMessage3 *>(static_cast<void *>(m_message.m_par3)))->~TemplateParameter3(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_4: (static_cast<tMessage4 *>(static_cast<void *>(m_message.m_par4)))->~TemplateParameter4(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_5: (static_cast<tMessage5 *>(static_cast<void *>(m_message.m_par5)))->~TemplateParameter5(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_6: (static_cast<tMessage6 *>(static_cast<void *>(m_message.m_par6)))->~TemplateParameter6(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_7: (static_cast<tMessage7 *>(static_cast<void *>(m_message.m_par7)))->~TemplateParameter7(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_8: (static_cast<tMessage8 *>(static_cast<void *>(m_message.m_par8)))->~TemplateParameter8(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
        }
        m_messageType = MT_none;
    }


    void setValue1(const tMessage1 &message)
    {
        setValue0();
        m_messageType = MT_1;
        new (m_message.m_par1)tMessage1(message); // placement new
    }

    void setValue2(const tMessage2 &message)
    {
        setValue0();
        m_messageType = MT_2;
        new (m_message.m_par2)tMessage2(message); // placement new
    }

    void setValue3(const tMessage3 &message)
    {
        setValue0();
        m_messageType = MT_3;
        new (m_message.m_par3)tMessage3(message); // placement new
    }

    void setValue4(const tMessage4 &message)
    {
        setValue0();
        m_messageType = MT_4;
        new (m_message.m_par4)tMessage4(message); // placement new
    }

    void setValue5(const tMessage5 &message)
    {
        setValue0();
        m_messageType = MT_5;
        new (m_message.m_par5)tMessage5(message); // placement new
    }

    void setValue6(const tMessage6 &message)
    {
        setValue0();
        m_messageType = MT_6;
        new (m_message.m_par6)tMessage6(message); // placement new
    }

    void setValue7(const tMessage7 &message)
    {
        setValue0();
        m_messageType = MT_7;
        new (m_message.m_par7)tMessage7(message); // placement new
    }

    void setValue8(const tMessage8 &message)
    {
        setValue0();
        m_messageType = MT_8;
        new (m_message.m_par8)tMessage8(message); // placement new
    }


    const tMessage1 &getValue1() const
    {
        assert(m_messageType == MT_1);
        return *(static_cast<const tMessage1 *>(static_cast<const void *>(m_message.m_par1)));
    }

    const tMessage2 &getValue2() const
    {
        assert(m_messageType == MT_2);
        return *(static_cast<const tMessage2 *>(static_cast<const void *>(m_message.m_par2)));
    }

    const tMessage3 &getValue3() const
    {
        assert(m_messageType == MT_3);
        return *(static_cast<const tMessage3 *>(static_cast<const void *>(m_message.m_par3)));
    }

    const tMessage4 &getValue4() const
    {
        assert(m_messageType == MT_4);
        return *(static_cast<const tMessage4 *>(static_cast<const void *>(m_message.m_par4)));
    }

    const tMessage5 &getValue5() const
    {
        assert(m_messageType == MT_5);
        return *(static_cast<const tMessage5 *>(static_cast<const void *>(m_message.m_par5)));
    }

    const tMessage6 &getValue6() const
    {
        assert(m_messageType == MT_6);
        return *(static_cast<const tMessage6 *>(static_cast<const void *>(m_message.m_par6)));
    }

    const tMessage7 &getValue7() const
    {
        assert(m_messageType == MT_7);
        return *(static_cast<const tMessage7 *>(static_cast<const void *>(m_message.m_par7)));
    }

    const tMessage8 &getValue8() const
    {
        assert(m_messageType == MT_8);
        return *(static_cast<const tMessage8 *>(static_cast<const void *>(m_message.m_par8)));
    }


}; // template class TMessageUnion8


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TEnpatcher8
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;
    typedef NSigslot::TSignature1<tMessageUnion> tOutputSignature;

    tOutputSignature::CSignal SIGNAL;

    void SLOT_nr1 (const tMessageUnion::tMessage1 &msg)
    {
        tMessageUnion mu;
        mu.setValue1 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr2 (const tMessageUnion::tMessage2 &msg)
    {
        tMessageUnion mu;
        mu.setValue2 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr3 (const tMessageUnion::tMessage3 &msg)
    {
        tMessageUnion mu;
        mu.setValue3 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr4 (const tMessageUnion::tMessage4 &msg)
    {
        tMessageUnion mu;
        mu.setValue4 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr5 (const tMessageUnion::tMessage5 &msg)
    {
        tMessageUnion mu;
        mu.setValue5 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr6 (const tMessageUnion::tMessage6 &msg)
    {
        tMessageUnion mu;
        mu.setValue6 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr7 (const tMessageUnion::tMessage7 &msg)
    {
        tMessageUnion mu;
        mu.setValue7 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr8 (const tMessageUnion::tMessage8 &msg)
    {
        tMessageUnion mu;
        mu.setValue8 (msg);
        SIGNAL (mu);
    }

}; // template class TEnpatcher8


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TDispatcher8
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;

    void operator() (const tMessageUnion &mu)
    {
        switch (mu.getMessageType())
        {
            case tMessageUnion::MT_none:   break;  // empty on purpose
            case tMessageUnion::MT_1:  SIGNAL_nr1(mu.getValue1()); break;
            case tMessageUnion::MT_2:  SIGNAL_nr2(mu.getValue2()); break;
            case tMessageUnion::MT_3:  SIGNAL_nr3(mu.getValue3()); break;
            case tMessageUnion::MT_4:  SIGNAL_nr4(mu.getValue4()); break;
            case tMessageUnion::MT_5:  SIGNAL_nr5(mu.getValue5()); break;
            case tMessageUnion::MT_6:  SIGNAL_nr6(mu.getValue6()); break;
            case tMessageUnion::MT_7:  SIGNAL_nr7(mu.getValue7()); break;
            case tMessageUnion::MT_8:  SIGNAL_nr8(mu.getValue8()); break;
        }
    }

    NSigslot::TSignature1<tMessageUnion::tMessage1>::CSignal SIGNAL_nr1;
    NSigslot::TSignature1<tMessageUnion::tMessage2>::CSignal SIGNAL_nr2;
    NSigslot::TSignature1<tMessageUnion::tMessage3>::CSignal SIGNAL_nr3;
    NSigslot::TSignature1<tMessageUnion::tMessage4>::CSignal SIGNAL_nr4;
    NSigslot::TSignature1<tMessageUnion::tMessage5>::CSignal SIGNAL_nr5;
    NSigslot::TSignature1<tMessageUnion::tMessage6>::CSignal SIGNAL_nr6;
    NSigslot::TSignature1<tMessageUnion::tMessage7>::CSignal SIGNAL_nr7;
    NSigslot::TSignature1<tMessageUnion::tMessage8>::CSignal SIGNAL_nr8;
}; // template class TDispatcher8



// ******************************************************************
// ******************************************************************
// ******************************************************************

/*!
    TMessageUnion<n> is a template class to store one Sigslot message
    out of a set of signatures.

    The template \c TMessageUnion<n> gets n parameters,
    \c tMessage1 to \c tMessage9.
    These classes may be of any class of value kind, but are
    supposed to be of type NSigslot::TSignature<...>::tMessage
    for a different kind of signatures.

    The TMessageUnion is simply a union for all these given classes
    and has an addidional enum to store, which type actually is stored.

*/
template <typename TemplateParameter1, typename TemplateParameter2, typename TemplateParameter3, typename TemplateParameter4, typename TemplateParameter5, typename TemplateParameter6, typename TemplateParameter7, typename TemplateParameter8, typename TemplateParameter9>
class TMessageUnion9
{
    public:

    /*
        Templates in general should typedef the template parameters
        to make them accessible by the user of the template class.
        At least, this is the opinion of Andrei Alexandrescu.
    */

    typedef TemplateParameter1 tMessage1;    //!< type of the 1st/nd/rd/th signal parameter
    typedef TemplateParameter2 tMessage2;    //!< type of the 2st/nd/rd/th signal parameter
    typedef TemplateParameter3 tMessage3;    //!< type of the 3st/nd/rd/th signal parameter
    typedef TemplateParameter4 tMessage4;    //!< type of the 4st/nd/rd/th signal parameter
    typedef TemplateParameter5 tMessage5;    //!< type of the 5st/nd/rd/th signal parameter
    typedef TemplateParameter6 tMessage6;    //!< type of the 6st/nd/rd/th signal parameter
    typedef TemplateParameter7 tMessage7;    //!< type of the 7st/nd/rd/th signal parameter
    typedef TemplateParameter8 tMessage8;    //!< type of the 8st/nd/rd/th signal parameter
    typedef TemplateParameter9 tMessage9;    //!< type of the 9st/nd/rd/th signal parameter

    enum {numberOfParameters = 9};    //!< number of the parameters of the signal

    typedef TMessageUnion9<tMessage1, tMessage2, tMessage3, tMessage4, tMessage5, tMessage6, tMessage7, tMessage8, tMessage9> tMessageUnion;

    /*!
        This enum is used to identify one of the n types.
    */

    enum tMessageType
    {
        MT_none = 0 //!< no message is stored

      , MT_1 = 1    //!< type number 1 (tMessage1) is stored
      , MT_2 = 2    //!< type number 2 (tMessage2) is stored
      , MT_3 = 3    //!< type number 3 (tMessage3) is stored
      , MT_4 = 4    //!< type number 4 (tMessage4) is stored
      , MT_5 = 5    //!< type number 5 (tMessage5) is stored
      , MT_6 = 6    //!< type number 6 (tMessage6) is stored
      , MT_7 = 7    //!< type number 7 (tMessage7) is stored
      , MT_8 = 8    //!< type number 8 (tMessage8) is stored
      , MT_9 = 9    //!< type number 9 (tMessage9) is stored

    };

    enum tMessageType getMessageType() const
    {
        return m_messageType;
    }


    private:

    enum tMessageType m_messageType; //!< stores, of which type the currently stored messag is.

    /*!
        The member variable \c m_message stores the selected message.
        See \m_messageType to detect, which member of the union is valid.
    */

    union
    {
        int m_dummy0;         //!< to get a proper alignment...
        long m_dummy1;        //!< to get a proper alignment...
        double m_dummy2;      //!< to get a proper alignment...
        void *m_dummy3;       //!< to get a proper alignment...
        unsigned char m_par1[sizeof(tMessage1)];   //!< the 1st/nd/rd/th possible type to store
        unsigned char m_par2[sizeof(tMessage2)];   //!< the 2st/nd/rd/th possible type to store
        unsigned char m_par3[sizeof(tMessage3)];   //!< the 3st/nd/rd/th possible type to store
        unsigned char m_par4[sizeof(tMessage4)];   //!< the 4st/nd/rd/th possible type to store
        unsigned char m_par5[sizeof(tMessage5)];   //!< the 5st/nd/rd/th possible type to store
        unsigned char m_par6[sizeof(tMessage6)];   //!< the 6st/nd/rd/th possible type to store
        unsigned char m_par7[sizeof(tMessage7)];   //!< the 7st/nd/rd/th possible type to store
        unsigned char m_par8[sizeof(tMessage8)];   //!< the 8st/nd/rd/th possible type to store
        unsigned char m_par9[sizeof(tMessage9)];   //!< the 9st/nd/rd/th possible type to store
    } m_message;


    public:

    TMessageUnion9()
    :
        m_messageType(MT_none)
    {
    }

    TMessageUnion9(const TMessageUnion9 &source)
    :
        m_messageType(MT_none)
    {
        *this = source;
    }
        
    TMessageUnion9 &operator= (const TMessageUnion9 &rhs)
    {
        if (this != &rhs)
        {
            setValue0();
            m_messageType = rhs.m_messageType;

        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: new (m_message.m_par1)tMessage1(*(static_cast<const tMessage1 *>(static_cast<const void *>(rhs.m_message.m_par1)))); break;  // placement new
            case MT_2: new (m_message.m_par2)tMessage2(*(static_cast<const tMessage2 *>(static_cast<const void *>(rhs.m_message.m_par2)))); break;  // placement new
            case MT_3: new (m_message.m_par3)tMessage3(*(static_cast<const tMessage3 *>(static_cast<const void *>(rhs.m_message.m_par3)))); break;  // placement new
            case MT_4: new (m_message.m_par4)tMessage4(*(static_cast<const tMessage4 *>(static_cast<const void *>(rhs.m_message.m_par4)))); break;  // placement new
            case MT_5: new (m_message.m_par5)tMessage5(*(static_cast<const tMessage5 *>(static_cast<const void *>(rhs.m_message.m_par5)))); break;  // placement new
            case MT_6: new (m_message.m_par6)tMessage6(*(static_cast<const tMessage6 *>(static_cast<const void *>(rhs.m_message.m_par6)))); break;  // placement new
            case MT_7: new (m_message.m_par7)tMessage7(*(static_cast<const tMessage7 *>(static_cast<const void *>(rhs.m_message.m_par7)))); break;  // placement new
            case MT_8: new (m_message.m_par8)tMessage8(*(static_cast<const tMessage8 *>(static_cast<const void *>(rhs.m_message.m_par8)))); break;  // placement new
            case MT_9: new (m_message.m_par9)tMessage9(*(static_cast<const tMessage9 *>(static_cast<const void *>(rhs.m_message.m_par9)))); break;  // placement new
        }

        }
        return *this;
    }

    ~TMessageUnion9()
    {
        setValue0();
    }


#if 0 // These constructors don't work, when the same type is used at several positions.
    explicit TMessageUnion9 (tMessageType type, const tMessage1 &message)
    :
        m_messageType(type),
    {
        m_message.m_par1 = message;
    }

    explicit TMessageUnion9 (tMessageType type, const tMessage2 &message)
    :
        m_messageType(type),
    {
        m_message.m_par2 = message;
    }

    explicit TMessageUnion9 (tMessageType type, const tMessage3 &message)
    :
        m_messageType(type),
    {
        m_message.m_par3 = message;
    }

    explicit TMessageUnion9 (tMessageType type, const tMessage4 &message)
    :
        m_messageType(type),
    {
        m_message.m_par4 = message;
    }

    explicit TMessageUnion9 (tMessageType type, const tMessage5 &message)
    :
        m_messageType(type),
    {
        m_message.m_par5 = message;
    }

    explicit TMessageUnion9 (tMessageType type, const tMessage6 &message)
    :
        m_messageType(type),
    {
        m_message.m_par6 = message;
    }

    explicit TMessageUnion9 (tMessageType type, const tMessage7 &message)
    :
        m_messageType(type),
    {
        m_message.m_par7 = message;
    }

    explicit TMessageUnion9 (tMessageType type, const tMessage8 &message)
    :
        m_messageType(type),
    {
        m_message.m_par8 = message;
    }

    explicit TMessageUnion9 (tMessageType type, const tMessage9 &message)
    :
        m_messageType(type),
    {
        m_message.m_par9 = message;
    }

#endif


    /*!
        \c setValue0() is used to delete any stored element.
    */
    void setValue0()
    {
        switch (m_messageType)
        {
            case MT_none:   break;  // empty on purpose
            case MT_1: (static_cast<tMessage1 *>(static_cast<void *>(m_message.m_par1)))->~TemplateParameter1(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_2: (static_cast<tMessage2 *>(static_cast<void *>(m_message.m_par2)))->~TemplateParameter2(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_3: (static_cast<tMessage3 *>(static_cast<void *>(m_message.m_par3)))->~TemplateParameter3(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_4: (static_cast<tMessage4 *>(static_cast<void *>(m_message.m_par4)))->~TemplateParameter4(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_5: (static_cast<tMessage5 *>(static_cast<void *>(m_message.m_par5)))->~TemplateParameter5(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_6: (static_cast<tMessage6 *>(static_cast<void *>(m_message.m_par6)))->~TemplateParameter6(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_7: (static_cast<tMessage7 *>(static_cast<void *>(m_message.m_par7)))->~TemplateParameter7(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_8: (static_cast<tMessage8 *>(static_cast<void *>(m_message.m_par8)))->~TemplateParameter8(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
            case MT_9: (static_cast<tMessage9 *>(static_cast<void *>(m_message.m_par9)))->~TemplateParameter9(); break; // The object has been constructed using placement new. To destruct it, the destructor has to be called directly without any delete operator. There is no placement delete.
        }
        m_messageType = MT_none;
    }


    void setValue1(const tMessage1 &message)
    {
        setValue0();
        m_messageType = MT_1;
        new (m_message.m_par1)tMessage1(message); // placement new
    }

    void setValue2(const tMessage2 &message)
    {
        setValue0();
        m_messageType = MT_2;
        new (m_message.m_par2)tMessage2(message); // placement new
    }

    void setValue3(const tMessage3 &message)
    {
        setValue0();
        m_messageType = MT_3;
        new (m_message.m_par3)tMessage3(message); // placement new
    }

    void setValue4(const tMessage4 &message)
    {
        setValue0();
        m_messageType = MT_4;
        new (m_message.m_par4)tMessage4(message); // placement new
    }

    void setValue5(const tMessage5 &message)
    {
        setValue0();
        m_messageType = MT_5;
        new (m_message.m_par5)tMessage5(message); // placement new
    }

    void setValue6(const tMessage6 &message)
    {
        setValue0();
        m_messageType = MT_6;
        new (m_message.m_par6)tMessage6(message); // placement new
    }

    void setValue7(const tMessage7 &message)
    {
        setValue0();
        m_messageType = MT_7;
        new (m_message.m_par7)tMessage7(message); // placement new
    }

    void setValue8(const tMessage8 &message)
    {
        setValue0();
        m_messageType = MT_8;
        new (m_message.m_par8)tMessage8(message); // placement new
    }

    void setValue9(const tMessage9 &message)
    {
        setValue0();
        m_messageType = MT_9;
        new (m_message.m_par9)tMessage9(message); // placement new
    }


    const tMessage1 &getValue1() const
    {
        assert(m_messageType == MT_1);
        return *(static_cast<const tMessage1 *>(static_cast<const void *>(m_message.m_par1)));
    }

    const tMessage2 &getValue2() const
    {
        assert(m_messageType == MT_2);
        return *(static_cast<const tMessage2 *>(static_cast<const void *>(m_message.m_par2)));
    }

    const tMessage3 &getValue3() const
    {
        assert(m_messageType == MT_3);
        return *(static_cast<const tMessage3 *>(static_cast<const void *>(m_message.m_par3)));
    }

    const tMessage4 &getValue4() const
    {
        assert(m_messageType == MT_4);
        return *(static_cast<const tMessage4 *>(static_cast<const void *>(m_message.m_par4)));
    }

    const tMessage5 &getValue5() const
    {
        assert(m_messageType == MT_5);
        return *(static_cast<const tMessage5 *>(static_cast<const void *>(m_message.m_par5)));
    }

    const tMessage6 &getValue6() const
    {
        assert(m_messageType == MT_6);
        return *(static_cast<const tMessage6 *>(static_cast<const void *>(m_message.m_par6)));
    }

    const tMessage7 &getValue7() const
    {
        assert(m_messageType == MT_7);
        return *(static_cast<const tMessage7 *>(static_cast<const void *>(m_message.m_par7)));
    }

    const tMessage8 &getValue8() const
    {
        assert(m_messageType == MT_8);
        return *(static_cast<const tMessage8 *>(static_cast<const void *>(m_message.m_par8)));
    }

    const tMessage9 &getValue9() const
    {
        assert(m_messageType == MT_9);
        return *(static_cast<const tMessage9 *>(static_cast<const void *>(m_message.m_par9)));
    }


}; // template class TMessageUnion9


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TEnpatcher9
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;
    typedef NSigslot::TSignature1<tMessageUnion> tOutputSignature;

    tOutputSignature::CSignal SIGNAL;

    void SLOT_nr1 (const tMessageUnion::tMessage1 &msg)
    {
        tMessageUnion mu;
        mu.setValue1 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr2 (const tMessageUnion::tMessage2 &msg)
    {
        tMessageUnion mu;
        mu.setValue2 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr3 (const tMessageUnion::tMessage3 &msg)
    {
        tMessageUnion mu;
        mu.setValue3 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr4 (const tMessageUnion::tMessage4 &msg)
    {
        tMessageUnion mu;
        mu.setValue4 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr5 (const tMessageUnion::tMessage5 &msg)
    {
        tMessageUnion mu;
        mu.setValue5 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr6 (const tMessageUnion::tMessage6 &msg)
    {
        tMessageUnion mu;
        mu.setValue6 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr7 (const tMessageUnion::tMessage7 &msg)
    {
        tMessageUnion mu;
        mu.setValue7 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr8 (const tMessageUnion::tMessage8 &msg)
    {
        tMessageUnion mu;
        mu.setValue8 (msg);
        SIGNAL (mu);
    }

    void SLOT_nr9 (const tMessageUnion::tMessage9 &msg)
    {
        tMessageUnion mu;
        mu.setValue9 (msg);
        SIGNAL (mu);
    }

}; // template class TEnpatcher9


// ******************************************************************

template <typename TemplateParameterMessageUnion>
class TDispatcher9
{
    public:

    typedef TemplateParameterMessageUnion tMessageUnion;

    void operator() (const tMessageUnion &mu)
    {
        switch (mu.getMessageType())
        {
            case tMessageUnion::MT_none:   break;  // empty on purpose
            case tMessageUnion::MT_1:  SIGNAL_nr1(mu.getValue1()); break;
            case tMessageUnion::MT_2:  SIGNAL_nr2(mu.getValue2()); break;
            case tMessageUnion::MT_3:  SIGNAL_nr3(mu.getValue3()); break;
            case tMessageUnion::MT_4:  SIGNAL_nr4(mu.getValue4()); break;
            case tMessageUnion::MT_5:  SIGNAL_nr5(mu.getValue5()); break;
            case tMessageUnion::MT_6:  SIGNAL_nr6(mu.getValue6()); break;
            case tMessageUnion::MT_7:  SIGNAL_nr7(mu.getValue7()); break;
            case tMessageUnion::MT_8:  SIGNAL_nr8(mu.getValue8()); break;
            case tMessageUnion::MT_9:  SIGNAL_nr9(mu.getValue9()); break;
        }
    }

    NSigslot::TSignature1<tMessageUnion::tMessage1>::CSignal SIGNAL_nr1;
    NSigslot::TSignature1<tMessageUnion::tMessage2>::CSignal SIGNAL_nr2;
    NSigslot::TSignature1<tMessageUnion::tMessage3>::CSignal SIGNAL_nr3;
    NSigslot::TSignature1<tMessageUnion::tMessage4>::CSignal SIGNAL_nr4;
    NSigslot::TSignature1<tMessageUnion::tMessage5>::CSignal SIGNAL_nr5;
    NSigslot::TSignature1<tMessageUnion::tMessage6>::CSignal SIGNAL_nr6;
    NSigslot::TSignature1<tMessageUnion::tMessage7>::CSignal SIGNAL_nr7;
    NSigslot::TSignature1<tMessageUnion::tMessage8>::CSignal SIGNAL_nr8;
    NSigslot::TSignature1<tMessageUnion::tMessage9>::CSignal SIGNAL_nr9;
}; // template class TDispatcher9



} // namespace NSigslot

#endif // NSigslot_TMessageUnion_generated_hpp_INCLUDEGUARD

/*
    end of file generated by the tool \c hpp_generator
*/
