/**
    \file TStandardCables.hpp
    \author Kurt Stege

    Defines the TSignature<n>-templates with its signals and slots.

    History:

    (ks) = Kurt Stege, kurt-stege@online.de

    2005-07-22 (ks):    File created.
    2005-07-25 (ks):    Current version.
*/

#ifndef NSigslot_TStandardCables_hpp_INCLUDEGUARD
#define NSigslot_TStandardCables_hpp_INCLUDEGUARD

/*
    Note: The header file TSignature.hpp is not directly required.
    The template TJunctionBox just depends on the given template
    parameter that provides the signature. So, the caller of the
    TJunctionBox already has to include TSignature.hpp for itself.
    And when the user provides some other matching Signature, the
    TJunctionBox will work as well.
*/
#include "Sigslot/TSignature.hpp"


namespace NSigslot {


// ******************************************************************

/*!
    TODO: Documentation for TCableToOperator.
*/

template <typename TemplateParameterSlot>
class TCableToOperator : public /* typename */ TemplateParameterSlot::tSignature::CSignal
{
    public:

    typedef TemplateParameterSlot tSlot;
    typedef typename tSlot::tSignature tSignature;
    typedef typename tSlot::tReceiver tReceiver;

    explicit TCableToOperator(tReceiver &receiver)
    :
        tSignature::CSignal (cableFunction_nonconst, &receiver)
    {
    }

    explicit TCableToOperator(const tReceiver &receiver)
    :
        tSignature::CSignal (cableFunction_const, &receiver)
    {
    }

    private:

    static void cableFunction_nonconst (void *context, const typename tSignature::CMessage &msg)
    {
        tReceiver *receiver = static_cast<tReceiver *>(context);
        tSlot::callOperator(*receiver, msg);
    }

    static void cableFunction_const (void *context, const typename tSignature::CMessage &msg)
    {
        const tReceiver *receiver = static_cast<const tReceiver *>(context);
        tSlot::callOperator(*receiver, msg);
    }
};


// ******************************************************************

/*!
    TODO: Documentation for TCableToMessageOperator.
*/

template <typename TemplateParameterSlot>
class TCableToMessageOperator : public /* typename */ TemplateParameterSlot::tSignature::CSignal
{
    public:

    typedef TemplateParameterSlot tSlot;
    typedef typename tSlot::tSignature tSignature;
    typedef typename tSlot::tReceiver tReceiver;

    explicit TCableToMessageOperator(tReceiver &receiver)
    :
        tSignature::CSignal (cableFunction_nonconst, &receiver)
    {
    }

    explicit TCableToMessageOperator(const tReceiver &receiver)
    :
        tSignature::CSignal (cableFunction_const, &receiver)
    {
    }

    private:

    static void cableFunction_nonconst (void *context, const typename tSignature::CMessage &msg)
    {
        tReceiver *receiver = static_cast<tReceiver *>(context);
        (*receiver)(msg);
    }

    static void cableFunction_const (void *context, const typename tSignature::CMessage &msg)
    {
        const tReceiver *receiver = static_cast<const tReceiver *>(context);
        (*receiver)(msg);
    }
};



/*!
    TODO: Documentation for TCableToSignal.
*/

template <typename TemplateParameterSignature>
class TCableToSignal : public /* typename */ TemplateParameterSignature::CSignal
{
    public:

    typedef TemplateParameterSignature tSignature;
    typedef typename tSignature::CSignal tSignal;

    explicit TCableToSignal(const tSignal &receiver)
    :
        tSignature::CSignal (cableFunction, &receiver)
    {
    }

    private:

    static void cableFunction (void *context, const typename tSignature::CMessage &msg)
    {
        const tSignal *receiver = static_cast<const tSignal *>(context);
        receiver->emitPackedMessage(msg);
    }

};


// ******************************************************************

/*!
    TODO: Documentation for TCableToMessageSignal.
*/

template <typename TemplateParameterSignature>
class TCableToMessageSignal : public /* typename */ TemplateParameterSignature::CSignal
{
    public:

    typedef TemplateParameterSignature tSignature;
    typedef typename tSignature::CMessage tMessage;
    typedef NSigslot::TSignature1<tMessage> tPackedMessageSignature;
    typedef tPackedMessageSignature::CSignal tPackedMessageSignal;

    explicit TCableToMessageSignal(const tPackedMessageSignal &receiver)
    :
        tSignature::CSignal (cableFunction, &receiver)
    {
    }

    private:

    static void cableFunction (void *context, const typename tSignature::CMessage &msg)
    {
        const tPackedMessageSignal *receiver = static_cast<const tPackedMessageSignal *>(context);
        *receiver(msg);
    }
};


} // namespace NSigslot

#endif // NSigslot_TStandardCables_hpp_INCLUDEGUARD
