/**
    \file CThread.cpp
    \author Kurt Stege

    Thread Component.

    Mainly used by Cogen to generate composite components.
    May be used manually for any purpose.

    History:

    (ks) = Kurt Stege, kurt-stege@online.de

    2005-08-23 (ks):    File created.
    2005-08-23 (ks):    Current version.
*/

#include "Sigslot/CThread.hpp"
#include "Base/CThread.hpp"

#include <assert.h>



class CLock
{
    public:

    CLock()
    {
        // TODO: Lock a global mutex
    }

    ~CLock()
    {
        // TODO: Unock the global mutex
    }
};


void NSigslot::CThread::start_mainloop (void *context)
{
    NSigslot::CThread *this_ = static_cast<NSigslot::CThread *>(context);
    this_->private_mainloop();
}


NSigslot::CThread::CThread()
:
    m_shutdownIsActive(false),
    m_threadCount(0)
{
}

NSigslot::CThread::~CThread()
{
    assert(m_threadCount == 0);
}


void NSigslot::CThread::SLOT_start()
{
    {
        CLock lock;
        if (m_shutdownIsActive)
        {
            return;
        }
        m_threadCount ++;
    }

    // TODO: start a new thread that calls start_mainloop()

    INIT_active(true);
    NBase::CThread::createThread (start_mainloop, this);
}

void NSigslot::CThread::INIT_shutdown()
{
    {
        CLock lock;
        m_shutdownIsActive = true;
        if (m_threadCount > 0)
        {
            return;
        }
    }
    INIT_active (false);
}

void NSigslot::CThread::private_mainloop()
{
    while (true)
    {
        {
            CLock lock;
            if (m_shutdownIsActive)
            {
                break;
            }
        }
        SIGNAL_thread();
    }

    {
        CLock lock;
        m_threadCount--;
    }
}


