/**
    \file CWait.cpp
    \author Kurt Stege

    Component that implements a blocking wait mechanism.

    Mainly used by Cogen to generate composite components.
    May be used manually for any purpose.

    History:

    (ks) = Kurt Stege, kurt-stege@online.de

    2005-08-24 (ks):    File created.
    2005-08-24 (ks):    Current version.
*/

#include "Sigslot/CWait.hpp"
#include "Base/CThread.hpp"

#include <assert.h>



class CLock
{
    public:

    CLock()
    {
        // TODO: Lock a global mutex
    }

    ~CLock()
    {
        // TODO: Unock the global mutex
    }
};


NSigslot::CWait::CWait()
:
    m_wakeupIsActive(false),
    m_shutdownIsActive(false)
{
}

NSigslot::CWait::~CWait()
{
}


void NSigslot::CWait::SLOT_work()
{
    /*
        Most times, the following loop runs just once, but in spurious
        timing situations a wakeup works for the wrong call to SLOT_work.
    */

    while (true)
    {
        bool wakeup;

        {
            CLock lock;
            if (m_shutdownIsActive)
            {
                return;
            }
            wakeup = m_wakeupIsActive;
            m_wakeupIsActive = false;
        }

        if (wakeup)
        {
            SIGNAL_work();
        }

        // TODO: blocking wait!
        NBase::CThread::sleep(1);
    }
}


void NSigslot::CWait::SLOT_wakeup()
{
    {
        CLock lock;
        m_wakeupIsActive = true;
    }
    // TODO: Wake up the blocking wait!
}


void NSigslot::CWait::INIT_shutdown()
{
    {
        CLock lock;
        m_shutdownIsActive = true;
    }
}

